*Generate Sample Data.sas;
*David A. Vandenbroucke, 8/26/2015 1:56 PM; 
%include 'C:\data\standard options.sas';
TITLE Generate Sample Data;
/*
     This program generates some sample data for testing the universal file flattener.
     The each set of SAS datasets includes:
          1. A master file (MASTER)
          2. A one-to-many file with a fixed number of replicates (CONSTANT)
          3. A one-to-many file with variable numbers of replicates (VARIABLE)
          4. another one-to-one file (ANOTHER)
          
     The program produces one such set for each of these:
          1. Single character key, character and numeric data
          2. Single numeric key, character and numeric data
          3. Two character key, character and numeric data
          4. Two numeric key, character and numeric data
          5. One character and one numeric key
          6. One character key, character data only
          7. One character key, numberic data only
          
     Each set is written to a different library.          
*/

/* USER CONFIGURATION SECTION ------------------------------------------------------*/
/* 
     INSTRUCTIONS:
     1. You must create directories named Set1 through Set7 with the same root path.
     2. Modify the RootPath definition below to point to the appropriate path.  DO NOT
          INCLUDE the trailing backslash.
*/

%LET RootPath = J:\REH\Dav\Data\CURRENT\Conference Notes\2016\SAS Global Forum\Data;

/* END OF USER CONFIGURATION SECTION -------------------------------------------------*/

%MACRO GenerateLibs;
     /* this macro generates the libnames for Set1-Set7. */
     %DO I = 1 %TO 7;
          LIBNAME Set&I. "&RootPath\Set&I.";
     %END;     /* of do i... */     
%MEND; /* generatelibs */

%GenerateLibs; /* create the libnames */

/* note that 65 is the ASCII code for 'A' */
DATA Set1.Master(DROP = _I_);
     LENGTH MCA MCB $4;
     DO _I_ = 65 TO 65+49; /* generate 50 records */
          CKey = BYTE(_I_);
          MNA = _I_;     /*numeric variable */
          MNB = 100*_I_; /* another numeric variable */
          MCA = CAT('M',BYTE(_I_),'A');     /* character variable */
          MCB = CAT('M',BYTE(_I_),'B');     /* another character variable */
          OUTPUT;
     END; /* do _i_...*/
RUN; /* Master */

DATA Set1.Constant (DROP = _I_ _J_);
     LENGTH CCA CCB $4;
     DO _I_ = 65 TO 65+49; /* generate 50 keys */
          CKey = BYTE(_I_);
          DO _J_ = 65 TO 65+2; /* generate three replicates */
               CNA = _J_;     /*numeric variable */
               CNB = 100*_J_; /* another numeric variable */
               CCA = CAT('C',BYTE(_I_),BYTE(_J_),'A');     /* character variable */
               CCB = CAT('C',BYTE(_I_),BYTE(_J_),'B');     /* another character variable */
               OUTPUT;
          END; /* do _J_...*/     
     END;
RUN; /* Constant */

DATA Set1.VARIABLE (DROP = _I_ _J_);
     LENGTH VCA VCB $4;
     DO _I_ = 65 TO 65+49; /* generate 50 keys */     
          CKey = BYTE(_I_);
          DO _J_ = 65 TO 65+MOD(_I_,10); /* generate up to 9 replicates */
               VNA = _J_;     /*numeric variable */
               VNB = 100*_J_; /* another numeric variable */
               VCA = CAT('V',BYTE(_I_),BYTE(_J_),'A');     /* character variable */
               VCB = CAT('V',BYTE(_I_),BYTE(_J_),'B');     /* another character variable */
               OUTPUT;
          END; /* do _J_...*/     
     END;
RUN; /* Variable */

DATA Set1.Another(DROP = _I_);
     LENGTH ACA ACB $4;
     DO _I_ = 65 TO 65+49; /* generate 50 records */
          CKey = BYTE(_I_);
          ANA = _I_;     /*numeric variable */
          ANB = 100*_I_; /* another numeric variable */
          ACA = CAT('A',BYTE(_I_),'A');     /* character variable */
          ACB = CAT('A',BYTE(_I_),'B');     /* another character variable */
          OUTPUT;
     END; /* do _i_...*/
RUN; /* Another */

/* ------------------------------------------------------------------------------------*/
/* Set 2:  Single numeric key */

/* note that 65 is the ASCII code for 'A' */
DATA Set2.Master(DROP = _I_);
     LENGTH MCA MCB $4;
     DO _I_ = 65 TO 65+49; /* generate 50 records */
          NKey = _I_;
          MNA = _I_;     /*numeric variable */
          MNB = 100*_I_; /* another numeric variable */
          MCA = CAT('M',BYTE(_I_),'A');     /* character variable */
          MCB = CAT('M',BYTE(_I_),'B');     /* another character variable */
          OUTPUT;
     END; /* do _i_...*/
RUN; /* Master */

DATA Set2.Constant (DROP = _I_ _J_);
     LENGTH CCA CCB $4;
     DO _I_ = 65 TO 65+49; /* generate 50 keys */
          NKey = _I_;
          DO _J_ = 65 TO 65+2; /* generate three replicates */
               CNA = _J_;     /*numeric variable */
               CNB = 100*_J_; /* another numeric variable */
               CCA = CAT('C',BYTE(_I_),BYTE(_J_),'A');     /* character variable */
               CCB = CAT('C',BYTE(_I_),BYTE(_J_),'B');     /* another character variable */
               OUTPUT;
          END; /* do _J_...*/     
     END;
RUN; /* Constant */

DATA Set2.VARIABLE (DROP = _I_ _J_);
     LENGTH VCA VCB $4;
     DO _I_ = 65 TO 65+49; /* generate 50 keys */     
          NKey = _I_;
          DO _J_ = 65 TO 65+MOD(_I_,10); /* generate up to 9 replicates */
               VNA = _J_;     /*numeric variable */
               VNB = 100*_J_; /* another numeric variable */
               VCA = CAT('V',BYTE(_I_),BYTE(_J_),'A');     /* character variable */
               VCB = CAT('V',BYTE(_I_),BYTE(_J_),'B');     /* another character variable */
               OUTPUT;
          END; /* do _J_...*/     
     END;
RUN; /* Variable */

DATA Set2.Another(DROP = _I_);
     LENGTH ACA ACB $4;
     DO _I_ = 65 TO 65+49; /* generate 50 records */
          NKey = _I_;
          ANA = _I_;     /*numeric variable */
          ANB = 100*_I_; /* another numeric variable */
          ACA = CAT('A',BYTE(_I_),'A');     /* character variable */
          ACB = CAT('A',BYTE(_I_),'B');     /* another character variable */
          OUTPUT;
     END; /* do _i_...*/
RUN; /* Another */

/* ------------------------------------------------------------------------------------*/
/* SET 3: Two character keys */

/* note that 65 is the ASCII code for 'A' */
DATA SET3.Master(DROP = _I_ _K_);
     LENGTH MCA MCB $4;
     DO _I_ = 65 TO 65+9; /* generate 10 keys */
          CKey = BYTE(_I_);
          DO _K_ = 97 TO 97+9;  /* generate 10 second keys */
               C2Key = BYTE(_K_);  /* second key */
               MNA = _I_;     /*numeric variable */
               MNB = _K_; /* another numeric variable */
               MCA = CAT('M',BYTE(_I_),'A');     /* character variable */
               MCB = CAT('M',BYTE(_K_),'B');     /* another character variable */
               OUTPUT;
          END; /* do _k_...*/
     END; /* do _i_...*/
RUN; /* Master */

DATA SET3.Constant (DROP = _I_ _J_ _K_);
     LENGTH CCA CCB $4;
     DO _I_ = 65 TO 65+9; /* generate 10 keys */
          CKey = BYTE(_I_);
          DO _K_ = 97 TO 97+9; /* generate 10 second keys */
               C2Key = BYTE(_K_);  /* second key */               
               DO _J_ = 65 TO 65+2; /* generate three replicates */
                    CNA = _J_;     /*numeric variable */
                    CNB = _K_+_I_; /* another numeric variable */
                    CCA = CAT('C',BYTE(_I_),BYTE(_J_),'A');     /* character variable */
                    CCB = CAT('C',BYTE(_K_),BYTE(_J_),'B');     /* another character variable */
                    OUTPUT;
               END; /* do _J_...*/  
          END; /* DO _k_...*/
     END;
RUN; /* Constant */

DATA SET3.VARIABLE (DROP = _I_ _J_ _K_);
     LENGTH VCA VCB $4;
     DO _I_ = 65 TO 65+9; /* generate 10 keys */     
          CKey = BYTE(_I_);
          DO _K_ = 97 to 97+9;     /* generate 10 second keys */
               C2Key = BYTE(_K_);
               DO _J_ = 65 TO 65+MOD(_I_,10); /* generate up to 9 replicates */
                    VNA = _J_;     /*numeric variable */
                    VNB = 100*_K_; /* another numeric variable */
                    VCA = CAT('V',BYTE(_I_),BYTE(_J_),'A');     /* character variable */
                    VCB = CAT('V',BYTE(_K_),BYTE(_J_),'B');     /* another character variable */
                    OUTPUT;
               END; /* do _J_...*/ 
          END; /* do _k_...*/
     END;
RUN; /* Variable */

DATA SET3.Another(DROP = _I_ _K_);
     LENGTH ACA ACB $4;
     DO _I_ = 65 TO 65+9; /* generate 10 keys */     
          CKey = BYTE(_I_);
          DO _K_ = 97 to 97+9;     /* generate 10 second keys */
               C2Key = BYTE(_K_);
               ANA = _I_;     /*numeric variable */
               ANB = 100*_K_; /* another numeric variable */
               ACA = CAT('A',BYTE(_I_),'A');     /* character variable */
               ACB = CAT('A',BYTE(_K_),'B');     /* another character variable */
               OUTPUT;
          END; /* do _k_...*/
     END; /* do _i_...*/
RUN; /* Another */

/* ------------------------------------------------------------------------------------*/
/* Set 4:  Two numeric keys */

/* note that 65 is the ASCII code for 'A' */
DATA SET4.Master(DROP = _I_ _K_);
     LENGTH MCA MCB $4;
     DO _I_ = 65 TO 65+9; /* generate 10 keys */
          NKey = _I_;
          DO _K_ = 97 TO 97+9;  /* generate 10 second keys */
               N2Key = _K_;  /* second key */
               MNA = _I_;     /*numeric variable */
               MNB = _K_; /* another numeric variable */
               MCA = CAT('M',BYTE(_I_),'A');     /* character variable */
               MCB = CAT('M',BYTE(_K_),'B');     /* another character variable */
               OUTPUT;
          END; /* do _k_...*/
     END; /* do _i_...*/
RUN; /* Master */

DATA SET4.Constant (DROP = _I_ _J_ _K_);
     LENGTH CCA CCB $4;
     DO _I_ = 65 TO 65+9; /* generate 10 keys */
          NKey = _I_;
          DO _K_ = 97 TO 97+9; /* generate 10 second keys */
               N2Key = _K_;  /* second key */               
               DO _J_ = 65 TO 65+2; /* generate three replicates */
                    CNA = _J_;     /*numeric variable */
                    CNB = _K_+_I_; /* another numeric variable */
                    CCA = CAT('C',BYTE(_I_),BYTE(_J_),'A');     /* character variable */
                    CCB = CAT('C',BYTE(_K_),BYTE(_J_),'B');     /* another character variable */
                    OUTPUT;
               END; /* do _J_...*/  
          END; /* DO _k_...*/
     END;
RUN; /* Constant */

DATA SET4.VARIABLE (DROP = _I_ _J_ _K_);
     LENGTH VCA VCB $4;
     DO _I_ = 65 TO 65+9; /* generate 10 keys */     
          NKey = _I_;
          DO _K_ = 97 to 97+9;     /* generate 10 second keys */
               N2Key = _K_;
               DO _J_ = 65 TO 65+MOD(_I_,10); /* generate up to 9 replicates */
                    VNA = _J_;     /*numeric variable */
                    VNB = 100*_K_; /* another numeric variable */
                    VCA = CAT('V',BYTE(_I_),BYTE(_J_),'A');     /* character variable */
                    VCB = CAT('V',BYTE(_K_),BYTE(_J_),'B');     /* another character variable */
                    OUTPUT;
               END; /* do _J_...*/ 
          END; /* do _k_...*/
     END;
RUN; /* Variable */

DATA SET4.Another(DROP = _I_ _K_);
     LENGTH ACA ACB $4;
     DO _I_ = 65 TO 65+9; /* generate 10 keys */     
          NKey = _I_;
          DO _K_ = 97 to 97+9;     /* generate 10 second keys */
               N2Key = _K_;
               ANA = _I_;     /*numeric variable */
               ANB = 100*_K_; /* another numeric variable */
               ACA = CAT('A',BYTE(_I_),'A');     /* character variable */
               ACB = CAT('A',BYTE(_K_),'B');     /* another character variable */
               OUTPUT;
          END; /* do _k_...*/
     END; /* do _i_...*/
RUN; /* Another */

/* ------------------------------------------------------------------------------------*/
/* Set 5:  One character and one numeric key */

/* note that 65 is the ASCII code for 'A' */
DATA SET5.Master(DROP = _I_ _K_);
     LENGTH MCA MCB $4;
     DO _I_ = 65 TO 65+9; /* generate 10 keys */
          NKey = _I_;
          DO _K_ = 97 TO 97+9;  /* generate 10 second keys */
               C2Key = BYTE(_K_);  /* second key */
               MNA = _I_;     /*numeric variable */
               MNB = _K_; /* another numeric variable */
               MCA = CAT('M',BYTE(_I_),'A');     /* character variable */
               MCB = CAT('M',BYTE(_K_),'B');     /* another character variable */
               OUTPUT;
          END; /* do _k_...*/
     END; /* do _i_...*/
RUN; /* Master */

DATA SET5.Constant (DROP = _I_ _J_ _K_);
     LENGTH CCA CCB $4;
     DO _I_ = 65 TO 65+9; /* generate 10 keys */
          NKey = _I_;
          DO _K_ = 97 TO 97+9; /* generate 10 second keys */
               C2Key = BYTE(_K_);  /* second key */               
               DO _J_ = 65 TO 65+2; /* generate three replicates */
                    CNA = _J_;     /*numeric variable */
                    CNB = _K_+_I_; /* another numeric variable */
                    CCA = CAT('C',BYTE(_I_),BYTE(_J_),'A');     /* character variable */
                    CCB = CAT('C',BYTE(_K_),BYTE(_J_),'B');     /* another character variable */
                    OUTPUT;
               END; /* do _J_...*/  
          END; /* DO _k_...*/
     END;
RUN; /* Constant */

DATA SET5.VARIABLE (DROP = _I_ _J_ _K_);
     LENGTH VCA VCB $4;
     DO _I_ = 65 TO 65+9; /* generate 10 keys */     
          NKey = _I_;
          DO _K_ = 97 to 97+9;     /* generate 10 second keys */
               C2Key = BYTE(_K_);
               DO _J_ = 65 TO 65+MOD(_I_,10); /* generate up to 9 replicates */
                    VNA = _J_;     /*numeric variable */
                    VNB = 100*_K_; /* another numeric variable */
                    VCA = CAT('V',BYTE(_I_),BYTE(_J_),'A');     /* character variable */
                    VCB = CAT('V',BYTE(_K_),BYTE(_J_),'B');     /* another character variable */
                    OUTPUT;
               END; /* do _J_...*/ 
          END; /* do _k_...*/
     END;
RUN; /* Variable */

DATA SET5.Another(DROP = _I_ _K_);
     LENGTH ACA ACB $4;
     DO _I_ = 65 TO 65+9; /* generate 10 keys */     
          NKey = _I_;
          DO _K_ = 97 to 97+9;     /* generate 10 second keys */
               C2Key = BYTE(_K_);
               ANA = _I_;     /*numeric variable */
               ANB = 100*_K_; /* another numeric variable */
               ACA = CAT('A',BYTE(_I_),'A');     /* character variable */
               ACB = CAT('A',BYTE(_K_),'B');     /* another character variable */
               OUTPUT;
          END; /* do _k_...*/
     END; /* do _i_...*/
RUN; /* Another */

/* Set 6:  Only character data */

DATA SET6.Another;
     SET Set1.Another (DROP=ACA ACB);
RUN;

DATA SET6.Constant;
     SET Set1.Constant (DROP=CCA CCB);
RUN;

DATA SET6.Master;
     SET Set1.Master (DROP=mCA mCB);
RUN;

DATA SET6.Variable;
     SET Set1.Variable (DROP=vCA vCB);
RUN;

/* Set 7:  Only numeric Data */
DATA SET7.Another;
     SET Set1.Another (DROP=ANA ANB);
RUN;

DATA SET7.Constant;
     SET Set1.Constant (DROP=CNA CNB);
RUN;

DATA SET7.Master;
     SET Set1.Master (DROP=mNA mNB);
RUN;

DATA SET7.Variable;
     SET Set1.Variable (DROP=vNA vNB);
RUN;

/* Show contents of all generated datasets */
%macro dumpcontents;
     %do Counter = 1 %to 7;
          proc datasets ddname=SET&COUNTER.;
           contents DATA=SET&COUNTER.._ALL_;
               quit;
          run;
     %end;
%mend;

%dumpcontents;

